@@ -8,6 +8,7 @@ use crate::python_interpreter::{InterpreterConfig, InterpreterKind, MINIMUM_PYTH
 use crate::{BuildContext, PythonInterpreter, Target};
 use anyhow::{bail, format_err, Context, Result};
 use cargo_metadata::{Metadata, Node};
+use cargo_options::heading;
 use pep440_rs::VersionSpecifiers;
 use serde::{Deserialize, Serialize};
 use std::collections::{HashMap, HashSet};
@@ -47,35 +48,45 @@ pub struct CargoOptions {
     pub quiet: bool,
 
     /// Number of parallel jobs, defaults to # of CPUs
-    #[arg(short = 'j', long, value_name = "N")]
+    #[arg(short = 'j', long, value_name = "N", help_heading = heading::COMPILATION_OPTIONS)]
     pub jobs: Option<usize>,
 
     /// Build artifacts with the specified Cargo profile
-    #[arg(long, value_name = "PROFILE-NAME")]
+    #[arg(long, value_name = "PROFILE-NAME", help_heading = heading::COMPILATION_OPTIONS)]
     pub profile: Option<String>,
 
     /// Space or comma separated list of features to activate
-    #[arg(short = 'F', long, action = clap::ArgAction::Append)]
+    #[arg(
+        short = 'F',
+        long,
+        action = clap::ArgAction::Append,
+        help_heading = heading::FEATURE_SELECTION,
+    )]
     pub features: Vec<String>,
 
     /// Activate all available features
-    #[arg(long)]
+    #[arg(long, help_heading = heading::FEATURE_SELECTION)]
     pub all_features: bool,
 
     /// Do not activate the `default` feature
-    #[arg(long)]
+    #[arg(long, help_heading = heading::FEATURE_SELECTION)]
     pub no_default_features: bool,
 
     /// Build for the target triple
-    #[arg(long, value_name = "TRIPLE", env = "CARGO_BUILD_TARGET")]
+    #[arg(
+        long,
+        value_name = "TRIPLE",
+        env = "CARGO_BUILD_TARGET",
+        help_heading = heading::COMPILATION_OPTIONS,
+    )]
     pub target: Option<String>,
 
     /// Directory for all generated artifacts
-    #[arg(long, value_name = "DIRECTORY")]
+    #[arg(long, value_name = "DIRECTORY", help_heading = heading::COMPILATION_OPTIONS)]
     pub target_dir: Option<PathBuf>,
 
     /// Path to Cargo.toml
-    #[arg(short = 'm', long, value_name = "PATH")]
+    #[arg(short = 'm', long, value_name = "PATH", help_heading = heading::MANIFEST_OPTIONS)]
     pub manifest_path: Option<PathBuf>,
 
     /// Ignore `rust-version` specification in packages
@@ -91,15 +102,15 @@ pub struct CargoOptions {
     pub color: Option<String>,
 
     /// Require Cargo.lock and cache are up to date
-    #[arg(long)]
+    #[arg(long, help_heading = heading::MANIFEST_OPTIONS)]
     pub frozen: bool,
 
     /// Require Cargo.lock is up to date
-    #[arg(long)]
+    #[arg(long, help_heading = heading::MANIFEST_OPTIONS)]
     pub locked: bool,
 
     /// Run without accessing the network
-    #[arg(long)]
+    #[arg(long, help_heading = heading::MANIFEST_OPTIONS)]
     pub offline: bool,
 
     /// Override a configuration value (unstable)
@@ -115,7 +126,8 @@ pub struct CargoOptions {
         long,
         value_name = "FMTS",
         value_delimiter = ',',
-        require_equals = true
+        require_equals = true,
+        help_heading = heading::COMPILATION_OPTIONS,
     )]
     pub timings: Option<Vec<String>>,
 
@@ -261,14 +273,14 @@ impl BuildOptions {
                             .get("EXT_SUFFIX")
                             .context("syconfig didn't define an `EXT_SUFFIX` ಠ_ಠ")?;
                         let soabi = sysconfig_data.get("SOABI");
-                        let abi_tag =
-                            soabi.and_then(|abi| abi.split('-').nth(1).map(ToString::to_string));
                         let interpreter_kind = soabi
                             .and_then(|tag| {
                                 if tag.starts_with("pypy") {
                                     Some(InterpreterKind::PyPy)
                                 } else if tag.starts_with("cpython") {
                                     Some(InterpreterKind::CPython)
+                                } else if tag.starts_with("graalpy") {
+                                    Some(InterpreterKind::GraalPy)
                                 } else {
                                     None
                                 }
@@ -281,7 +293,6 @@ impl BuildOptions {
                                 interpreter_kind,
                                 abiflags,
                                 ext_suffix: ext_suffix.to_string(),
-                                abi_tag,
                                 pointer_width: None,
                             },
                             executable: PathBuf::new(),
@@ -349,7 +360,6 @@ impl BuildOptions {
                                 interpreter_kind: InterpreterKind::CPython,
                                 abiflags: "".to_string(),
                                 ext_suffix: ".pyd".to_string(),
-                                abi_tag: None,
                                 pointer_width: None,
                             },
                             executable: PathBuf::new(),
@@ -376,7 +386,6 @@ impl BuildOptions {
                                 interpreter_kind: InterpreterKind::CPython,
                                 abiflags: "".to_string(),
                                 ext_suffix: ".pyd".to_string(),
-                                abi_tag: None,
                                 pointer_width: None,
                             },
                             executable: PathBuf::new(),
@@ -416,7 +425,6 @@ impl BuildOptions {
                                 interpreter_kind: InterpreterKind::CPython,
                                 abiflags: "".to_string(),
                                 ext_suffix: "".to_string(),
-                                abi_tag: None,
                                 pointer_width: None,
                             },
                             executable: PathBuf::new(),
@@ -507,29 +515,27 @@ impl BuildOptions {
 
         let mut universal2 = target_triple.as_deref() == Some("universal2-apple-darwin");
         // Also try to determine universal2 from ARCHFLAGS environment variable
-        if let Ok(arch_flags) = env::var("ARCHFLAGS") {
-            let arches: HashSet<&str> = arch_flags
-                .split("-arch")
-                .filter_map(|x| {
-                    let x = x.trim();
-                    if x.is_empty() {
-                        None
-                    } else {
-                        Some(x)
-                    }
-                })
-                .collect();
-            match (arches.contains("x86_64"), arches.contains("arm64")) {
-                (true, true) => universal2 = true,
-                (true, false) if target_triple.is_none() => {
-                    target_triple = Some("x86_64-apple-darwin".to_string())
+        if target_triple.is_none() {
+            if let Ok(arch_flags) = env::var("ARCHFLAGS") {
+                let arches: HashSet<&str> = arch_flags
+                    .split("-arch")
+                    .filter_map(|x| {
+                        let x = x.trim();
+                        if x.is_empty() {
+                            None
+                        } else {
+                            Some(x)
+                        }
+                    })
+                    .collect();
+                match (arches.contains("x86_64"), arches.contains("arm64")) {
+                    (true, true) => universal2 = true,
+                    (true, false) => target_triple = Some("x86_64-apple-darwin".to_string()),
+                    (false, true) => target_triple = Some("aarch64-apple-darwin".to_string()),
+                    (false, false) => {}
                 }
-                (false, true) if target_triple.is_none() => {
-                    target_triple = Some("aarch64-apple-darwin".to_string())
-                }
-                _ => {}
-            }
-        };
+            };
+        }
         if universal2 {
             target_triple = None;
         }
@@ -557,12 +563,14 @@ impl BuildOptions {
                 if cfg!(test) {
                     match env::var_os("MATURIN_TEST_PYTHON") {
                         Some(python) => vec![python.into()],
-                        None => vec![PathBuf::from("python3")],
+                        None => vec![target.get_python()],
                     }
                 } else {
-                    vec![PathBuf::from("python3")]
+                    vec![target.get_python()]
                 }
             } else {
+                // XXX: False positive clippy warning
+                #[allow(clippy::redundant_clone)]
                 self.interpreter.clone()
             };
             self.find_interpreters(&bridge, &interpreter, &target, None, generate_import_lib)?
@@ -593,7 +601,7 @@ impl BuildOptions {
                     x.compatibility()
                 })
                 .or(if use_zig {
-                    if target.is_musl_target() {
+                    if target.is_musl_libc() {
                         // Zig bundles musl 1.2
                         Some(PlatformTag::Musllinux { x: 1, y: 2 })
                     } else {
@@ -603,7 +611,7 @@ impl BuildOptions {
                     }
                 } else {
                     // Defaults to musllinux_1_2 for musl target if it's not bin bindings
-                    if target.is_musl_target() && !bridge.is_bin() {
+                    if target.is_musl_libc() && !bridge.is_bin() {
                         Some(PlatformTag::Musllinux { x: 1, y: 2 })
                     } else {
                         None
@@ -621,6 +629,8 @@ impl BuildOptions {
         for platform_tag in &platform_tags {
             if !platform_tag.is_supported() {
                 eprintln!("⚠️  Warning: {platform_tag} is unsupported by the Rust compiler.");
+            } else if platform_tag.is_musllinux() && !target.is_musl_libc() {
+                eprintln!("⚠️  Warning: {target} is not compatible with {platform_tag}.");
             }
         }
 
@@ -688,7 +698,7 @@ fn validate_bridge_type(
     match bridge {
         BridgeModel::Bin(None) => {
             // Only support two different kind of platform tags when compiling to musl target without any binding crates
-            if platform_tags.iter().any(|tag| tag.is_musllinux()) && !target.is_musl_target() {
+            if platform_tags.iter().any(|tag| tag.is_musllinux()) && !target.is_musl_libc() {
                 bail!(
                     "Cannot mix musllinux and manylinux platform tags when compiling to {}",
                     target.target_triple()
@@ -744,7 +754,7 @@ fn filter_cargo_targets(
         .iter()
         .filter(|target| match bridge {
             BridgeModel::Bin(_) => {
-                let is_bin = target.kind.contains(&"bin".to_string());
+                let is_bin = target.is_bin();
                 if target.required_features.is_empty() {
                     is_bin
                 } else {
@@ -1101,8 +1111,7 @@ fn find_interpreter_in_host(
     requires_python: Option<&VersionSpecifiers>,
 ) -> Result<Vec<PythonInterpreter>> {
     let interpreters = if !interpreter.is_empty() {
-        PythonInterpreter::check_executables(interpreter, target, bridge)
-            .context("The given list of python interpreters is invalid")?
+        PythonInterpreter::check_executables(interpreter, target, bridge)?
     } else {
         PythonInterpreter::find_all(target, bridge, requires_python)
             .context("Finding python interpreters failed")?
@@ -1132,6 +1141,11 @@ fn find_interpreter_in_sysconfig(
         let python = interp.display().to_string();
         let (python_impl, python_ver) = if let Some(ver) = python.strip_prefix("pypy") {
             (InterpreterKind::PyPy, ver.strip_prefix('-').unwrap_or(ver))
+        } else if let Some(ver) = python.strip_prefix("graalpy") {
+            (
+                InterpreterKind::GraalPy,
+                ver.strip_prefix('-').unwrap_or(ver),
+            )
         } else if let Some(ver) = python.strip_prefix("python") {
             (
                 InterpreterKind::CPython,
@@ -1146,7 +1160,12 @@ fn find_interpreter_in_sysconfig(
             // Eg: -i 3.9 without interpreter kind, assume it's CPython
             (InterpreterKind::CPython, &*python)
         } else {
-            bail!("Unsupported Python interpreter: {}", python);
+            // if interpreter not known
+            if std::path::Path::new(&python).is_file() {
+                bail!("Python interpreter should be a kind of interpreter (e.g. 'python3.8' or 'pypy3.9') when cross-compiling, got path to interpreter: {}", python);
+            } else {
+                bail!("Unsupported Python interpreter for cross-compilation: {}; supported interpreters are pypy, graalpy, and python (cpython)", python);
+            }
         };
         if python_ver.is_empty() {
             continue;
