@@ -4,6 +4,7 @@
 //! Run with --help for usage information
 
 use anyhow::{bail, Context, Result};
+use cargo_options::heading;
 #[cfg(feature = "zig")]
 use cargo_zigbuild::Zig;
 #[cfg(feature = "cli-completion")]
@@ -12,8 +13,8 @@ use clap::{Parser, Subcommand};
 #[cfg(feature = "scaffolding")]
 use maturin::{ci::GenerateCI, init_project, new_project, GenerateProjectOptions};
 use maturin::{
-    develop, write_dist_info, BridgeModel, BuildOptions, CargoOptions, PathWriter, PlatformTag,
-    PythonInterpreter, Target,
+    develop, write_dist_info, BridgeModel, BuildOptions, CargoOptions, DevelopOptions, PathWriter,
+    PlatformTag, PythonInterpreter, Target,
 };
 #[cfg(feature = "upload")]
 use maturin::{upload_ui, PublishOpt};
@@ -26,7 +27,8 @@ use tracing::debug;
     version,
     name = env!("CARGO_PKG_NAME"),
     display_order = 1,
-    after_help = "Visit https://maturin.rs to learn more about maturin."
+    after_help = "Visit https://maturin.rs to learn more about maturin.",
+    styles = cargo_options::styles(),
 )]
 #[cfg_attr(feature = "cargo-clippy", allow(clippy::large_enum_variant))]
 /// Build and publish crates with pyo3, rust-cpython and cffi bindings as well
@@ -36,7 +38,7 @@ enum Opt {
     /// Build the crate into python packages
     Build {
         /// Build artifacts in release mode, with optimizations
-        #[arg(short = 'r', long)]
+        #[arg(short = 'r', long, help_heading = heading::COMPILATION_OPTIONS)]
         release: bool,
         /// Strip the library for minimum file size
         #[arg(long)]
@@ -73,36 +75,7 @@ enum Opt {
     },
     #[command(name = "develop", alias = "dev")]
     /// Install the crate as module in the current virtualenv
-    ///
-    /// Note that this command doesn't create entrypoints
-    Develop {
-        /// Which kind of bindings to use
-        #[arg(
-            short = 'b',
-            long = "bindings",
-            alias = "binding-crate",
-            value_parser = ["pyo3", "pyo3-ffi", "rust-cpython", "cffi", "uniffi", "bin"]
-        )]
-        bindings: Option<String>,
-        /// Pass --release to cargo
-        #[arg(short = 'r', long)]
-        release: bool,
-        /// Strip the library for minimum file size
-        #[arg(long)]
-        strip: bool,
-        /// Install extra requires aka. optional dependencies
-        ///
-        /// Use as `--extras=extra1,extra2`
-        #[arg(
-            short = 'E',
-            long,
-            value_delimiter = ',',
-            action = clap::ArgAction::Append
-        )]
-        extras: Vec<String>,
-        #[command(flatten)]
-        cargo_options: CargoOptions,
-    },
+    Develop(DevelopOptions),
     /// Build only a source distribution (sdist) without compiling.
     ///
     /// Building a source distribution requires a pyproject.toml with a `[build-system]` table.
@@ -368,7 +341,7 @@ fn run() -> Result<()> {
         #[cfg(feature = "upload")]
         Opt::Publish {
             build,
-            publish,
+            mut publish,
             debug,
             no_strip,
             no_sdist,
@@ -387,6 +360,7 @@ fn run() -> Result<()> {
             }
 
             let items = wheels.into_iter().map(|wheel| wheel.0).collect::<Vec<_>>();
+            publish.non_interactive_on_ci();
 
             upload_ui(&items, &publish)?
         }
@@ -404,16 +378,10 @@ fn run() -> Result<()> {
                 eprintln!(" - {interpreter}");
             }
         }
-        Opt::Develop {
-            bindings,
-            release,
-            strip,
-            extras,
-            cargo_options,
-        } => {
-            let target = Target::from_target_triple(cargo_options.target.clone())?;
+        Opt::Develop(develop_options) => {
+            let target = Target::from_target_triple(develop_options.cargo_options.target.clone())?;
             let venv_dir = detect_venv(&target)?;
-            develop(bindings, cargo_options, &venv_dir, release, strip, extras)?;
+            develop(develop_options, &venv_dir)?;
         }
         Opt::SDist { manifest_path, out } => {
             let build_options = BuildOptions {
@@ -437,11 +405,12 @@ fn run() -> Result<()> {
         #[cfg(feature = "scaffolding")]
         Opt::GenerateCI(generate_ci) => generate_ci.execute()?,
         #[cfg(feature = "upload")]
-        Opt::Upload { publish, files } => {
+        Opt::Upload { mut publish, files } => {
             if files.is_empty() {
                 eprintln!("⚠️  Warning: No files given, exiting.");
                 return Ok(());
             }
+            publish.non_interactive_on_ci();
 
             upload_ui(&files, &publish)?
         }
